@@ -12,22 +12,30 @@ class Teradata(Dialect):
     TYPED_DIVISION = True
 
     TIME_MAPPING = {
-        "Y": "%Y",
-        "YYYY": "%Y",
         "YY": "%y",
-        "MMMM": "%B",
-        "MMM": "%b",
-        "DD": "%d",
-        "D": "%-d",
-        "HH": "%H",
-        "H": "%-H",
-        "MM": "%M",
+        "Y4": "%Y",
+        "YYYY": "%Y",
+        "M4": "%B",
+        "M3": "%b",
         "M": "%-M",
-        "SS": "%S",
+        "MI": "%M",
+        "MM": "%m",
+        "MMM": "%b",
+        "MMMM": "%B",
+        "D": "%-d",
+        "DD": "%d",
+        "D3": "%j",
+        "DDD": "%j",
+        "H": "%-H",
+        "HH": "%H",
+        "HH24": "%H",
         "S": "%-S",
+        "SS": "%S",
         "SSSSSS": "%f",
         "E": "%a",
         "EE": "%a",
+        "E3": "%a",
+        "E4": "%A",
         "EEE": "%a",
         "EEEE": "%A",
     }
