@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import datetime
 import inspect
 import logging
 import re
@@ -9,13 +10,13 @@ from collections.abc import Collection
 from contextlib import contextmanager
 from copy import copy
 from enum import Enum
+from itertools import count
 
 if t.TYPE_CHECKING:
     from sqlglot import exp
+    from sqlglot._typing import A, E, T
     from sqlglot.expressions import Expression
 
-    T = t.TypeVar("T")
-    E = t.TypeVar("E", bound=Expression)
 
 CAMEL_CASE_PATTERN = re.compile("(?<!^)(?=[A-Z])")
 PYTHON_VERSION = sys.version_info[:2]
@@ -23,12 +24,26 @@ logger = logging.getLogger("sqlglot")
 
 
 class AutoName(Enum):
-    """This is used for creating enum classes where `auto()` is the string form of the corresponding value's name."""
+    """
+    This is used for creating Enum classes where `auto()` is the string form
+    of the corresponding enum's identifier (e.g. FOO.value results in "FOO").
 
-    def _generate_next_value_(name, _start, _count, _last_values):  # type: ignore
+    Reference: https://docs.python.org/3/howto/enum.html#using-automatic-values
+    """
+
+    def _generate_next_value_(name, _start, _count, _last_values):
         return name
 
 
+class classproperty(property):
+    """
+    Similar to a normal property but works for class methods
+    """
+
+    def __get__(self, obj: t.Any, owner: t.Any = None) -> t.Any:
+        return classmethod(self.fget).__get__(None, owner)()  # type: ignore
+
+
 def seq_get(seq: t.Sequence[T], index: int) -> t.Optional[T]:
     """Returns the value in `seq` at position `index`, or `None` if `index` is out of bounds."""
     try:
@@ -52,7 +67,7 @@ def ensure_list(value):
     Ensures that a value is a list, otherwise casts or wraps it into one.
 
     Args:
-        value: the value of interest.
+        value: The value of interest.
 
     Returns:
         The value cast as a list if it's a list or a tuple, or else the value wrapped in a list.
@@ -80,7 +95,7 @@ def ensure_collection(value):
     Ensures that a value is a collection (excluding `str` and `bytes`), otherwise wraps it into a list.
 
     Args:
-        value: the value of interest.
+        value: The value of interest.
 
     Returns:
         The value if it's a collection, or else the value wrapped in a list.
@@ -92,13 +107,13 @@ def ensure_collection(value):
     )
 
 
-def csv(*args, sep: str = ", ") -> str:
+def csv(*args: str, sep: str = ", ") -> str:
     """
     Formats any number of string arguments as CSV.
 
     Args:
-        args: the string arguments to format.
-        sep: the argument separator.
+        args: The string arguments to format.
+        sep: The argument separator.
 
     Returns:
         The arguments formatted as a CSV string.
@@ -115,9 +130,9 @@ def subclasses(
     Returns all subclasses for a collection of classes, possibly excluding some of them.
 
     Args:
-        module_name: the name of the module to search for subclasses in.
-        classes: class(es) we want to find the subclasses of.
-        exclude: class(es) we want to exclude from the returned list.
+        module_name: The name of the module to search for subclasses in.
+        classes: Class(es) we want to find the subclasses of.
+        exclude: Class(es) we want to exclude from the returned list.
 
     Returns:
         The target subclasses.
@@ -133,20 +148,20 @@ def subclasses(
 
 def apply_index_offset(
     this: exp.Expression,
-    expressions: t.List[t.Optional[E]],
+    expressions: t.List[E],
     offset: int,
-) -> t.List[t.Optional[E]]:
+) -> t.List[E]:
     """
     Applies an offset to a given integer literal expression.
 
     Args:
-        this: the target of the index
-        expressions: the expression the offset will be applied to, wrapped in a list.
-        offset: the offset that will be applied.
+        this: The target of the index.
+        expressions: The expression the offset will be applied to, wrapped in a list.
+        offset: The offset that will be applied.
 
     Returns:
         The original expression with the offset applied to it, wrapped in a list. If the provided
-        `expressions` argument contains more than one expressions, it's returned unaffected.
+        `expressions` argument contains more than one expression, it's returned unaffected.
     """
     if not offset or len(expressions) != 1:
         return expressions
@@ -166,15 +181,12 @@ def apply_index_offset(
     ):
         return expressions
 
-    if expression:
-        if not expression.type:
-            annotate_types(expression)
-        if t.cast(exp.DataType, expression.type).this in exp.DataType.INTEGER_TYPES:
-            logger.warning("Applying array index offset (%s)", offset)
-            expression = simplify(
-                exp.Add(this=expression.copy(), expression=exp.Literal.number(offset))
-            )
-            return [expression]
+    if not expression.type:
+        annotate_types(expression)
+    if t.cast(exp.DataType, expression.type).this in exp.DataType.INTEGER_TYPES:
+        logger.warning("Applying array index offset (%s)", offset)
+        expression = simplify(exp.Add(this=expression, expression=exp.Literal.number(offset)))
+        return [expression]
 
     return expressions
 
@@ -189,8 +201,8 @@ def while_changing(expression: Expression, func: t.Callable[[Expression], E]) ->
     Applies a transformation to a given expression until a fix point is reached.
 
     Args:
-        expression: the expression to be transformed.
-        func: the transformation to be applied.
+        expression: The expression to be transformed.
+        func: The transformation to be applied.
 
     Returns:
         The transformed expression.
@@ -198,6 +210,7 @@ def while_changing(expression: Expression, func: t.Callable[[Expression], E]) ->
     while True:
         for n, *_ in reversed(tuple(expression.walk())):
             n._hash = hash(n)
+
         start = hash(expression)
         expression = func(expression)
 
@@ -205,37 +218,40 @@ def while_changing(expression: Expression, func: t.Callable[[Expression], E]) ->
             n._hash = None
         if start == hash(expression):
             break
+
     return expression
 
 
-def tsort(dag: t.Dict[T, t.List[T]]) -> t.List[T]:
+def tsort(dag: t.Dict[T, t.Set[T]]) -> t.List[T]:
     """
     Sorts a given directed acyclic graph in topological order.
 
     Args:
-        dag: the graph to be sorted.
+        dag: The graph to be sorted.
 
     Returns:
         A list that contains all of the graph's nodes in topological order.
     """
     result = []
 
-    def visit(node: T, visited: t.Set[T]) -> None:
-        if node in result:
-            return
-        if node in visited:
+    for node, deps in tuple(dag.items()):
+        for dep in deps:
+            if not dep in dag:
+                dag[dep] = set()
+
+    while dag:
+        current = {node for node, deps in dag.items() if not deps}
+
+        if not current:
             raise ValueError("Cycle error")
 
-        visited.add(node)
+        for node in current:
+            dag.pop(node)
 
-        for dep in dag.get(node, []):
-            visit(dep, visited)
+        for deps in dag.values():
+            deps -= current
 
-        visited.remove(node)
-        result.append(node)
-
-    for node in dag:
-        visit(node, set())
+        result.extend(sorted(current))  # type: ignore
 
     return result
 
@@ -259,7 +275,7 @@ def csv_reader(read_csv: exp.ReadCSV) -> t.Any:
     Returns a csv reader given the expression `READ_CSV(name, ['delimiter', '|', ...])`.
 
     Args:
-        read_csv: a `ReadCSV` function call
+        read_csv: A `ReadCSV` function call.
 
     Yields:
         A python csv reader.
@@ -268,7 +284,7 @@ def csv_reader(read_csv: exp.ReadCSV) -> t.Any:
     file = open_file(read_csv.name)
 
     delimiter = ","
-    args = iter(arg.name for arg in args)
+    args = iter(arg.name for arg in args)  # type: ignore
     for k, v in zip(args, args):
         if k == "delimiter":
             delimiter = v
@@ -286,8 +302,8 @@ def find_new_name(taken: t.Collection[str], base: str) -> str:
     Searches for a new name.
 
     Args:
-        taken: a collection of taken names.
-        base: base name to alter.
+        taken: A collection of taken names.
+        base: Base name to alter.
 
     Returns:
         The new, available name.
@@ -304,9 +320,18 @@ def find_new_name(taken: t.Collection[str], base: str) -> str:
     return new
 
 
+def name_sequence(prefix: str) -> t.Callable[[], str]:
+    """Returns a name generator given a prefix (e.g. a0, a1, a2, ... if the prefix is "a")."""
+    sequence = count()
+    return lambda: f"{prefix}{next(sequence)}"
+
+
 def object_to_dict(obj: t.Any, **kwargs) -> t.Dict:
     """Returns a dictionary created from an object's attributes."""
-    return {**{k: copy(v) for k, v in vars(obj).copy().items()}, **kwargs}
+    return {
+        **{k: v.copy() if hasattr(v, "copy") else copy(v) for k, v in vars(obj).items()},
+        **kwargs,
+    }
 
 
 def split_num_words(
@@ -316,10 +341,10 @@ def split_num_words(
     Perform a split on a value and return N words as a result with `None` used for words that don't exist.
 
     Args:
-        value: the value to be split.
-        sep: the value to use to split on.
-        min_num_words: the minimum number of words that are going to be in the result.
-        fill_from_start: indicates that if `None` values should be inserted at the start or end of the list.
+        value: The value to be split.
+        sep: The value to use to split on.
+        min_num_words: The minimum number of words that are going to be in the result.
+        fill_from_start: Indicates that if `None` values should be inserted at the start or end of the list.
 
     Examples:
         >>> split_num_words("db.table", ".", 3)
@@ -349,12 +374,14 @@ def is_iterable(value: t.Any) -> bool:
         False
 
     Args:
-        value: the value to check if it is an iterable.
+        value: The value to check if it is an iterable.
 
     Returns:
         A `bool` value indicating if it is an iterable.
     """
-    return hasattr(value, "__iter__") and not isinstance(value, (str, bytes))
+    from sqlglot import Expression
+
+    return hasattr(value, "__iter__") and not isinstance(value, (str, bytes, Expression))
 
 
 def flatten(values: t.Iterable[t.Iterable[t.Any] | t.Any]) -> t.Iterator[t.Any]:
@@ -369,7 +396,7 @@ def flatten(values: t.Iterable[t.Iterable[t.Any] | t.Any]) -> t.Iterator[t.Any]:
         [1, 2, 3]
 
     Args:
-        values: the value to be flattened.
+        values: The value to be flattened.
 
     Yields:
         Non-iterable elements in `values`.
@@ -385,7 +412,7 @@ def dict_depth(d: t.Dict) -> int:
     """
     Get the nesting depth of a dictionary.
 
-    For example:
+    Example:
         >>> dict_depth(None)
         0
         >>> dict_depth({})
@@ -396,12 +423,6 @@ def dict_depth(d: t.Dict) -> int:
         2
         >>> dict_depth({"a": {"b": {}}})
         3
-
-    Args:
-        d (dict): dictionary
-
-    Returns:
-        int: depth
     """
     try:
         return 1 + dict_depth(next(iter(d.values())))
@@ -414,25 +435,56 @@ def dict_depth(d: t.Dict) -> int:
 
 
 def first(it: t.Iterable[T]) -> T:
-    """Returns the first element from an iterable.
-
-    Useful for sets.
-    """
+    """Returns the first element from an iterable (useful for sets)."""
     return next(i for i in it)
 
 
-def should_identify(text: str, identify: str | bool) -> bool:
-    """Checks if text should be identified given an identify option.
-
-    Args:
-        text: the text to check.
-        identify: "always" | True - always returns true, "safe" - true if no upper case
-
-    Returns:
-        Whether or not a string should be identified.
+def merge_ranges(ranges: t.List[t.Tuple[A, A]]) -> t.List[t.Tuple[A, A]]:
     """
-    if identify is True or identify == "always":
+    Merges a sequence of ranges, represented as tuples (low, high) whose values
+    belong to some totally-ordered set.
+
+    Example:
+        >>> merge_ranges([(1, 3), (2, 6)])
+        [(1, 6)]
+    """
+    if not ranges:
+        return []
+
+    ranges = sorted(ranges)
+
+    merged = [ranges[0]]
+
+    for start, end in ranges[1:]:
+        last_start, last_end = merged[-1]
+
+        if start <= last_end:
+            merged[-1] = (last_start, max(last_end, end))
+        else:
+            merged.append((start, end))
+
+    return merged
+
+
+def is_iso_date(text: str) -> bool:
+    try:
+        datetime.date.fromisoformat(text)
         return True
-    if identify == "safe":
-        return not any(char.isupper() for char in text)
-    return False
+    except ValueError:
+        return False
+
+
+def is_iso_datetime(text: str) -> bool:
+    try:
+        datetime.datetime.fromisoformat(text)
+        return True
+    except ValueError:
+        return False
+
+
+# Interval units that operate on date components
+DATE_UNITS = {"day", "week", "month", "quarter", "year", "year_month"}
+
+
+def is_date_unit(expression: t.Optional[exp.Expression]) -> bool:
+    return expression is not None and expression.name.lower() in DATE_UNITS
