@@ -1,10 +1,10 @@
 from sqlglot import exp
 from sqlglot.optimizer.normalize import normalized
-from sqlglot.optimizer.scope import build_scope
+from sqlglot.optimizer.scope import build_scope, find_in_scope
 from sqlglot.optimizer.simplify import simplify
 
 
-def pushdown_predicates(expression):
+def pushdown_predicates(expression, dialect=None):
     """
     Rewrite sqlglot AST to pushdown predicates in FROMS and JOINS
 
@@ -36,22 +36,28 @@ def pushdown_predicates(expression):
                     if isinstance(parent, exp.Join) and parent.side == "RIGHT":
                         selected_sources = {k: (node, source)}
                         break
-                pushdown(where.this, selected_sources, scope_ref_count)
+                pushdown(where.this, selected_sources, scope_ref_count, dialect)
 
             # joins should only pushdown into itself, not to other joins
             # so we limit the selected sources to only itself
             for join in select.args.get("joins") or []:
-                name = join.this.alias_or_name
-                pushdown(join.args.get("on"), {name: scope.selected_sources[name]}, scope_ref_count)
+                name = join.alias_or_name
+                if name in scope.selected_sources:
+                    pushdown(
+                        join.args.get("on"),
+                        {name: scope.selected_sources[name]},
+                        scope_ref_count,
+                        dialect,
+                    )
 
     return expression
 
 
-def pushdown(condition, sources, scope_ref_count):
+def pushdown(condition, sources, scope_ref_count, dialect):
     if not condition:
         return
 
-    condition = condition.replace(simplify(condition))
+    condition = condition.replace(simplify(condition, dialect=dialect))
     cnf_like = normalized(condition) or not normalized(condition, dnf=True)
 
     predicates = list(
@@ -78,7 +84,11 @@ def pushdown_cnf(predicates, scope, scope_ref_count):
                 break
             if isinstance(node, exp.Select):
                 predicate.replace(exp.true())
-                node.where(replace_aliases(node, predicate), copy=False)
+                inner_predicate = replace_aliases(node, predicate)
+                if find_in_scope(inner_predicate, exp.AggFunc):
+                    node.having(inner_predicate, copy=False)
+                else:
+                    node.where(inner_predicate, copy=False)
 
 
 def pushdown_dnf(predicates, scope, scope_ref_count):
@@ -93,10 +103,10 @@ def pushdown_dnf(predicates, scope, scope_ref_count):
     pushdown_tables = set()
 
     for a in predicates:
-        a_tables = set(exp.column_table_names(a))
+        a_tables = exp.column_table_names(a)
 
         for b in predicates:
-            a_tables &= set(exp.column_table_names(b))
+            a_tables &= exp.column_table_names(b)
 
         pushdown_tables.update(a_tables)
 
@@ -139,7 +149,11 @@ def pushdown_dnf(predicates, scope, scope_ref_count):
             if isinstance(node, exp.Join):
                 node.on(predicate, copy=False)
             elif isinstance(node, exp.Select):
-                node.where(replace_aliases(node, predicate), copy=False)
+                inner_predicate = replace_aliases(node, predicate)
+                if find_in_scope(inner_predicate, exp.AggFunc):
+                    node.having(inner_predicate, copy=False)
+                else:
+                    node.where(inner_predicate, copy=False)
 
 
 def nodes_for_predicate(predicate, sources, scope_ref_count):
@@ -147,7 +161,7 @@ def nodes_for_predicate(predicate, sources, scope_ref_count):
     tables = exp.column_table_names(predicate)
     where_condition = isinstance(predicate.find_ancestor(exp.Join, exp.Where), exp.Where)
 
-    for table in tables:
+    for table in sorted(tables):
         node, source = sources.get(table) or (None, None)
 
         # if the predicate is in a where statement we can try to push it down
