@@ -201,7 +201,7 @@ class TestSchema(unittest.TestCase):
     def test_schema_normalization(self):
         schema = MappingSchema(
             schema={"x": {"`y`": {"Z": {"a": "INT", "`B`": "VARCHAR"}, "w": {"C": "INT"}}}},
-            dialect="spark",
+            dialect="clickhouse",
         )
 
         table_z = exp.Table(this="z", db="y", catalog="x")
@@ -216,16 +216,64 @@ class TestSchema(unittest.TestCase):
         # Check that add_table normalizes both the table and the column names to be added / updated
         schema = MappingSchema()
         schema.add_table("Foo", {"SomeColumn": "INT", '"SomeColumn"': "DOUBLE"})
-        self.assertEqual(schema.column_names(exp.Table(this="fOO")), ["somecolumn", "SomeColumn"])
+        self.assertEqual(schema.column_names(exp.table_("fOO")), ["somecolumn", "SomeColumn"])
 
         # Check that names are normalized to uppercase for Snowflake
         schema = MappingSchema(schema={"x": {"foo": "int", '"bLa"': "int"}}, dialect="snowflake")
-        self.assertEqual(schema.column_names(exp.Table(this="x")), ["FOO", "bLa"])
+        self.assertEqual(schema.column_names(exp.table_("x")), ["FOO", "bLa"])
 
         # Check that switching off the normalization logic works as expected
         schema = MappingSchema(schema={"x": {"foo": "int"}}, normalize=False, dialect="snowflake")
-        self.assertEqual(schema.column_names(exp.Table(this="x")), ["foo"])
+        self.assertEqual(schema.column_names(exp.table_("x")), ["foo"])
 
         # Check that the correct dialect is used when calling schema methods
+        # Note: T-SQL is case-insensitive by default, so `fo` in clickhouse will match the normalized table name
         schema = MappingSchema(schema={"[Fo]": {"x": "int"}}, dialect="tsql")
-        self.assertEqual(schema.column_names("[Fo]"), schema.column_names("`Fo`", dialect="spark"))
+        self.assertEqual(
+            schema.column_names("[Fo]"), schema.column_names("`fo`", dialect="clickhouse")
+        )
+
+        # Check that all column identifiers are normalized to lowercase for BigQuery, even quoted
+        # ones. Also, ensure that tables aren't normalized, since they're case-sensitive by default.
+        schema = MappingSchema(schema={"Foo": {"`BaR`": "int"}}, dialect="bigquery")
+        self.assertEqual(schema.column_names("Foo"), ["bar"])
+        self.assertEqual(schema.column_names("foo"), [])
+
+        # Check that the schema's normalization setting can be overridden
+        schema = MappingSchema(schema={"X": {"y": "int"}}, normalize=False, dialect="snowflake")
+        self.assertEqual(schema.column_names("x", normalize=True), ["y"])
+
+    def test_same_number_of_qualifiers(self):
+        schema = MappingSchema({"x": {"y": {"c1": "int"}}})
+
+        with self.assertRaises(SchemaError) as ctx:
+            schema.add_table("z", {"c2": "int"})
+
+        self.assertEqual(
+            str(ctx.exception),
+            "Table z must match the schema's nesting level: 2.",
+        )
+
+        schema = MappingSchema()
+        schema.add_table("x.y", {"c1": "int"})
+
+        with self.assertRaises(SchemaError) as ctx:
+            schema.add_table("z", {"c2": "int"})
+
+        self.assertEqual(
+            str(ctx.exception),
+            "Table z must match the schema's nesting level: 2.",
+        )
+
+        with self.assertRaises(SchemaError) as ctx:
+            MappingSchema({"x": {"y": {"c1": "int"}}, "z": {"c2": "int"}})
+
+        self.assertEqual(
+            str(ctx.exception),
+            "Table z must match the schema's nesting level: 2.",
+        )
+
+    def test_has_column(self):
+        schema = MappingSchema({"x": {"c": "int"}})
+        self.assertTrue(schema.has_column("x", exp.column("c")))
+        self.assertFalse(schema.has_column("x", exp.column("k")))
